<?php
/**
 * Blocks class.
 *
 * @package Iconic_Woo_Bundled_Products
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Iconic_WBP_Blocks class.
 */
class Iconic_WBP_Blocks {
	/**
	 * Run.
	 */
	public static function run() {
		add_filter( 'render_block_woocommerce/single-product', array( __CLASS__, 'override_single_product_block_render' ), 20, 3 );
		add_filter( 'render_block_woocommerce/product-price', array( __CLASS__, 'override_single_product_price_block_render' ), 20, 3 );
	}

	/**
	 * Override single product block render.
	 *
	 * @param string   $block_content The block content.
	 * @param array    $block         The full block, including name and attributes.
	 * @param WP_Block $instance      The block instance.
	 * 
	 * @return string
	 */
	public static function override_single_product_block_render( $block_content, $block, $block_instance ) {
		global $iconic_woo_bundled_products;

		if ( is_admin() ) {
			return $block_content;
		}

		if ( empty( $block['attrs']['productId'] ) ) {
			return $block_content;
		}

		ob_start();
		$iconic_woo_bundled_products->output_bundled_products( $block['attrs']['productId'] );
		$wbp_html = ob_get_clean();

		// 6 characters is the length of the closing </div>.
		return substr_replace( $block_content, $wbp_html, strlen( $block_content ) - 6, 0 );
	}

	/**
	 * Override single product price block render.
	 *
	 * @param string   $block_content The block content.
	 * @param array    $block         The full block, including name and attributes.
	 * @param WP_Block $instance      The block instance.
	 * 
	 * @return string
	 */
	public static function override_single_product_price_block_render( $block_content, $block, $block_instance ) {
		global $post;

		if ( is_admin() ) {
			return $block_content;
		}

		if ( empty( $block['attrs']['isDescendentOfSingleProductBlock'] ) ) {
			return $block_content;
		}

		$product_id = self::get_single_product_block_product_id( $post );

		if ( ! $product_id ) {
			return $block_content;
		}

		$block_product = new WC_Product_Bundled( $product_id );

		if ( ! $block_product ) {
			return $block_content;
		}
		
		return $block_product->get_price_html();
	}

	/**
	 * Extract the product ID from the WC single product block.
	 *
	 * @param WP_Post $post WP_Post object instance.
	 * 
	 * @return bool|int
	 */
	public static function get_single_product_block_product_id( $post ) {
		$product_id = false;

		if ( $post && has_block( 'woocommerce/single-product', $post ) ) {
			foreach( parse_blocks( $post->post_content ) as $block ) {
				if ( 
					'woocommerce/single-product' === $block['blockName'] &&
					! empty( $block['attrs']['productId'] )
				) {
					$product_id = absint( $block['attrs']['productId'] );
					break;
				}
			}
		}

		return $product_id;
	}
}
